/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is from autoform.
 *
 * The Initial Developer of the Original Code is
 * Michael Kessler, Bluewin AG <michael.kessler@team.bluewin.com>.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Jay <sprokitthemonkey@hotmail.com>
 * Balabanov Dmitry <Balabanov@iso.ru>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
//************************************************/
// action class constructor
//************************************************/

function actions() {
	
   // public methods
   this.doHelp = doHelp;
   this.doDelete = doDelete;
   this.doReset = doReset;
   this.doLoad = doLoad;
   this.doLoadForm = doLoadForm;
   this.doSave = doSave;
   this.doSaveForm = doSaveForm;
   this.doVisualize = doVisualize;
   this.doRestore = doRestore;
   this.desDecode = desDecode;
   this.desEncode = desEncode;
   
   // private properties
   var changedElements = new Array();
   
   var count = 0;
   var masterpassword = null;
   
   var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
   var sbSrv = Components.classes['@mozilla.org/intl/stringbundle;1'].createInstance();
   var sbSrvI = sbSrv.QueryInterface(Components.interfaces.nsIStringBundleService);
   var autoformBundle = sbSrvI.createBundle("chrome://autoform/locale/autoform.properties");
   
   //************************************************/
   // display help
   //************************************************/
   
   function doHelp() {
   	  try {
			      var browser = getBrowser();
			      var tab = browser.addTab('chrome://autoform/locale/help.html', getReferrer(document));
			      browser.selectedTab = tab;

   		 } catch(e) {
   		    debug(e, "doHelp()");
   		 }  
   }
   
   //************************************************/
   // delete form preferences
   //************************************************/
   
   function doDelete() {
   	  try {
			      if(confirm(autoformBundle.GetStringFromName("deleteAll"))) {
			         var pageForms = window.content.document.forms;
			         
			         // loop through all forms
			         for(var i = 0; i < pageForms.length; i++) {
			            var BranchObj = getAutoformBranch(pageForms[i].name);
			            BranchObj.deleteBranch("");
			            prefService.savePrefFile(null);
			         }
			         
			         doReset();
			      }

   		 } catch(e) {
   		    debug(e, "doDelete()");
   		 }  
   }
   
   //************************************************/
   // reset forms
   //************************************************/
   
   function doReset() {
   	  try {
			      var pageForms = window.content.document.forms;
			      
			      // loop through all forms
			      for(var i = 0; i < pageForms.length; i++) {
			         pageForms[i].reset();
			      }
   		 } catch(e) {
   		    debug(e, "doReset()");
   		 }  
   }
   
   //************************************************/
   // decide autoload
   //************************************************/
   
   function doLoad() {
   	  try {
			      if(preferences.autoload) {
			         // load form automatically, check if not to load a page explicit
			         if(!isUrlMatch(preferences.loadNeverArray)) {
			            doLoadForm();
			         }
			      }
			      else {
			         // do not load form automatically, check if to load a page explicit
			         if(isUrlMatch(preferences.loadAlwaysArray)) {
			            doLoadForm();
			         }
			      }

   		 } catch(e) {
   		    debug(e, "doLoad()");
   		 }  
   }
   
   //************************************************/
   // load data into form
   //************************************************/
   
   function doLoadForm() {
   	  try {
			      changedElements = new Array();
			      count = 0;
			      
			      // check for frames
			      if (!window.content) {
			      	   return;
			      }
			      
			      var myFrames = window.content.document.getElementsByTagName("frame");
			      if(myFrames.length == 0) {
			         // no frames, get forms
			         var pageForms = window.content.document.getElementsByTagName("form");
			      }
			      else {
			         var pageForms = new Array();
			         var formCount = 0;
			         for(var fr = 0; fr < myFrames.length; fr++) {
			         	  if (myFrames[fr].contentDocument) {
						            myForms = myFrames[fr].contentDocument.getElementsByTagName("form");
						            for(var fo = 0; fo < myForms.length; fo++) {
			   			            pageForms[formCount++] = myForms[fo];
			      			      }
			         	  }
			         }
			      }
			      
			      // loop through all forms
			      for(var i = 0; i < pageForms.length; i++) {
			      	
			         // get form name
			         var formName = getInternalElementId(pageForms[i], "form", i);
			         
			         if(formName != "") {
			         	
			            var formElements = pageForms[i].elements;
			            llPref = getAutoformBranch(formName);
			            
			            // loop through all elememts
			            for(var j = 0; j < formElements.length; j++) {
			         
			               // get attributes
			               if (formElements[j].type) {
						               var elmType = formElements[j].type.toLowerCase();
			               } else {
			               	   var elmType = "";
			               }
			               // get element name and value
			               var elmName = getInternalElementId(formElements[j], "element", j);
			               var elmValue = formElements[j].value;
			             
			               if(elmName != "" && elmType != "") {
			
			          								var elementId = doHexEncode(elmName + "@@@" + elmType);
			
			                  // Check to see if the form element has been saved before
			                  if(llPref.prefHasUserValue(elementId)) {
			             
			                     if(elmType == "text" || elmType == "password" || elmType == "textarea" || elmType == "file") {
			                        // handle text, password, textarea & file input elements
			                        if(elmValue != "" || preferences.overwrite == true) {
			                           if(elmType == "password") {
			                              if(preferences.encryptpasswords) {
			                                 //var kt = desDecode(llPref.getCharPref(elmName));
			                                 //formElements[j].value = kt.substring(0, kt.indexOf("\0"));
			                                 formElements[j].value = desDecode(llPref.getCharPref(elementId));
			                              }
			                              else {
			                                 formElements[j].value = doHexDecode(llPref.getCharPref(elementId));
			                              }
			                           }
			                           else {
			                              formElements[j].value = doHexDecode(llPref.getCharPref(elementId));
			                           }
			                           changedElements[count++] = formElements[j];
			                        }
			                     }
			                     else if(elmType == "checkbox") {
			                        // handle checkbox
			                        formElements[j].checked = llPref.getBoolPref(elementId);
			                        changedElements[count++] = formElements[j];
			                     }
			                     else if(elmType == "radio") {
			                        // handle radio
			                        var check = doHexDecode(llPref.getCharPref(elementId));
			                        if(formElements[j].value == check) {
			                           formElements[j].checked = true;
			                        }
			                        else {
			                           formElements[j].checked = false;
			                        }
			                        changedElements[count++] = formElements[j];
			                     }
			                     else if(elmType == "select-one" || elmType == "select-multiple") {
			                        // handle select box
			                        var selection = doHexDecode(llPref.getCharPref(elementId));
			                        var el = formElements[j];
			                        // first clear all
			                        for(var e = 0; e < el.length; e++) {
			                           el[e].selected = false;
			                        }
			                        // loop through all options elements
			                        for(var e = 0; e < el.length; e++) {
			                           // this is a tokenizer
			                           var pos = selection.indexOf('|@|');
			                           var last = 0;
			                           while(pos != - 1) {
			                              var check = selection.substring(last, pos);
			                              if(check == el[e].value) {
			                                 el[e].selected = true;
			                              }
			                              last = pos + 3;
			                              pos = selection.indexOf('|@|', last);
			                           }
			                        }
			                        changedElements[count++] = formElements[j];
			                     }
			                  }
			               }
			            }
			         }
			      }
			      
			      if(preferences.visualize) {
			         doVisualize();
			      }
      
   		 } catch(e) {
   		    debug(e, "doLoadForm()");
   		 }  
   }
     
   //************************************************/
   // decide autosave
   //************************************************/
   
   function doSave() {
   	  try {
			      if(preferences.autosave) {
			         // save form automatically, check if not to save a page explicit
			         if(!isUrlMatch(preferences.saveNeverArray)) {
			            doSaveForm();
			         }
			      }
			      else {
			         // do not save form automatically, check if to save a page explicit
			         if(isUrlMatch(preferences.saveAlwaysArray)) {
			            doSaveForm();
			         }
			      }
   		 } catch(e) {
   		    debug(e, "doSave()");
   		 }  
   }
   
   //************************************************/
   // save form values
   //************************************************/
   
   function doSaveForm() {
   	  try {
			      if(window.content.document == null) {
			         return;
			      }
			      changedElements = new Array();
			      count = 0;
			      // check frames
			      var myFrames = window.content.document.getElementsByTagName("frame");
			      if(myFrames.length == 0) {
			         // no frames, get forms
			         var pageForms = window.content.document.getElementsByTagName("form");
			      }
			      else {
			         var pageForms = new Array();
			         var formCount = 0;
			         for(var fr = 0; fr < myFrames.length; fr++) {
			         	  if (myFrames[fr].contentDocument) {
   			            myForms = myFrames[fr].contentDocument.getElementsByTagName("form");
			               for(var fo = 0; fo < myForms.length; fo++) {
			                  pageForms[formCount++] = myForms[fo];
			               }
			         	  }
			         }
			      }
			      // loop through all forms
			      for(var i = 0; i < pageForms.length; i++) {
			         // get form name
			         var formName = getInternalElementId(pageForms[i], "form", i);
			         
			         if(formName != "") {
			         	
			            var formElements = pageForms[i].elements;
			            llPref = getAutoformBranch(formName);
			            
			            // loop through all elements
			            for(var j = 0; j < formElements.length; j++) {
			            	
			               var elmType = formElements[j].type.toLowerCase();
			               
			               // get element name and value
			               var elmName = getInternalElementId(formElements[j], "element", j);
			               
			               if(elmName != "") {
			               	
			               			var elementId = doHexEncode(elmName + "@@@" + elmType);
			                  var elmValue = formElements[j].value;
			                  
			                  // handle text, textarea & file input boxes
			                  if(elmType == "text" || elmType == "textarea" || elmType == "file") {
			                     if(elmValue != "" || preferences.emptyvalues == true) {
			                        llPref.setCharPref(elementId , doHexEncode(elmValue));
			                        changedElements[count++] = formElements[j];
			                     }
			                  }
			                  else if(elmType == "password") {
			                     // handle password
			                     if(elmValue != "" || preferences.emptyvalues == true) {
			                        if(preferences.savepasswords) {
			                           if(preferences.encryptpasswords) {
			                              llPref.setCharPref(elementId, desEncode(elmValue));
			                           }
			                           else {
			                              llPref.setCharPref(elementId, doHexEncode(elmValue));
			                           }
			                           changedElements[count++] = formElements[j];
			                        }
			                     }
			                  }
			                  else if(elmType == "checkbox") {
			                     // handle checkbox
			                     if(formElements[j].checked) {
			                        llPref.setBoolPref(elementId, 1);
			                     }
			                     else {
			                        llPref.setBoolPref(elementId, 0);
			                     }
			                     changedElements[count++] = formElements[j];
			                  }
			                  else if((elmType == "radio") &&(formElements[j].checked)) {
			                     // handle radio
			                     llPref.setCharPref(elementId, doHexEncode(elmValue));
			                     changedElements[count++] = formElements[j];
			                  }
			                  else if(elmType == "select-one" || elmType == "select-multiple") {
			                     // handle select
			                     var selection = "";
			                     var el = formElements[j];
			                     // save all values
			                     for(var e = 0; e < el.length; e++) {
			                        if(el[e].selected) {
			                           selection += el[e].value + "|@|";
			                        }
			                     }
			                     llPref.setCharPref(elementId, doHexEncode(selection));
			                     changedElements[count++] = formElements[j];
			                  }
			               }
			            }
			         }
			      }
			
			      if(preferences.visualize) {
			         doVisualize();
			      }  

   		 } catch(e) {
   		    debug(e, "doSaveForm()");
   		 }         	
   }
   
   //************************************************/
   // Returns the internal Element id
   //************************************************/
   
   function getInternalElementId(theElement, elementdef, number) {
   			try {
			      var elementId = "";

			      if(theElement.name != null && theElement.name != "") {
			         elementId = theElement.name;
			      }

			      if(theElement.id != null && theElement.id != "") {
			         elementId += theElement.id;
			      }

			      if(elementId == "") {
			         elementId = elementdef + "_number_" + number;
			      }

			      return elementId;

   		 } catch(e) {
   		    debug(e, "getInternalElementId()");
   		 }       
   }
   
   //************************************************/
   // Do visualisation of form load /save
   //************************************************/
   
   function doVisualize() {
						try {
			      for(i = 0; i < changedElements.length; i++) {
			         changedElements[i].setAttribute("autochanged", "true");
			      }
			
									// create stylesheet link
							  var linkElm = window.content.document.createElement("link");
							  linkElm.setAttribute("id", "autoformVisualize");
							  linkElm.setAttribute("href", "chrome://autoform/skin/visualize.css");
							  linkElm.setAttribute("rel", "stylesheet");
							  linkElm.setAttribute("type", "text/css");
							  
							  // attach it to the page
							  window.content.document.documentElement.childNodes[0].appendChild(linkElm);
							  
			      setTimeout("actions.doRestore()", 250);

   		 } catch(e) {
   		    debug(e, "doVisualize()");
   		 }   			
   }

   //************************************************/
   // Do restore
   //************************************************/
   
   function doRestore() {
						try {
			      
			      for(i = 0; i < changedElements.length; i++) {
			         changedElements[i].setAttribute("autochanged", "false");
			      }

					    css = window.content.document.getElementById("autoformVisualize");
					    
					    if (css != null) {
					    	  css.parentNode.removeChild(css);
					    }

   		 } catch(e) {
   		    debug(e, "doRestore()");
   		 }   			
   }

   //************************************************/
   // Decode encrypted text
   //************************************************/
   
   function desDecode(text) {
   			try {
			      if(masterpassword == null) {
			         getMasterPassword();
			      }

			      if(masterpassword == null) {
			         alert(autoformBundle.GetStringFromName("noMasterPassword"));
			         return null;
			      } else {
						      return des(masterpassword, doHexDecode(text), 0);
			      }
			      
   		 } catch(e) {
   		    debug(e, "desDecode()");
   		 }   			
   }
   
   //************************************************/
   // Encode encrypted text
   //************************************************/
   
   function desEncode(text) {
						try {
		      if(masterpassword == null) {
		         getMasterPassword();
		      }

		      if(masterpassword == null) {
		         alert(autoformBundle.GetStringFromName("noMasterPassword"));
			        return null;
		      } else {		      
   		      return doHexEncode(des(masterpassword, text, 1));
		      }

   		 } catch(e) {
   		    debug(e, "desEncode()");
   		 }   			
   }
   
   //************************************************/
   // set master password for encryption
   //************************************************/
   
   function getMasterPassword() {
   	  try {
			      // get a reference to the prompt service component.
			      var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
			      var pwdObj = new Object();
			      var checkObj = new Object();

			      if(promptService.promptPassword(window, autoformBundle.GetStringFromName("masterpassword"), autoformBundle.GetStringFromName("entermasterpassword"), pwdObj, null, checkObj)) {
			         masterpassword = pwdObj.value;
			      }

   		 } catch(e) {
   		    debug(e, "getMasterPassword()");
   		 }   			
   }
   
   //************************************************/
   // return the branch where to store/get the data
   //************************************************/
   
   function getAutoformBranch(theForm) {
   	  try {
			      var theBranch = "autoform." + preferences.autouser + "." + "formdata.";
			      // Hostname
			      if(preferences.hashhostname) {
			         theBranch = theBranch + doHexEncode(window.content.document.location.hostname) + ".";
			      }
			      // path
			      if(preferences.hashpathname) {
			         var thePathname = window.content.document.location.pathname;
			         if(thePathname == null || thePathname == "") {
			            thePathname = "/";
			         }
			         theBranch = theBranch + doHexEncode(thePathname) + ".";
			      }
			      // querystring
			      if(preferences.hashsearch) {
			         var theQuerystring = excludeParam(window.content.document.location.search);
			         if(theQuerystring == null || theQuerystring == "") {
			            theQuerystring = "?";
			         }
			         theBranch = theBranch + doHexEncode(theQuerystring) + ".";
			      }
			      // the form
			      if(preferences.hashform) {
			         theBranch = theBranch + doHexEncode(theForm) + ".";
			      }
			      return prefService.getBranch(theBranch);

   		 } catch(e) {
   		    debug(e, "getAutoformBranch()");
   		 }   			
   }
   
   //************************************************/
   // Exclude parameters
   //************************************************/

   function excludeParam(exclude) {
   		 try {
   		 	  if (preferences.excludeParametersArray != null) {
						   			for (i = 0; i < preferences.excludeParametersArray.length; i++) {
						   						var start = exclude.indexOf(preferences.excludeParametersArray[i] + "=");
						   						if (start != -1) {
						   									var end = exclude.indexOf("&", start);
						   									if (end == -1) {
						   									   exclude = exclude.substring(0, start);
						   									   if (exclude.substring(exclude.length-1) == "&") {
						   									   			exclude = exclude.substring(0, exclude.length-1);
						   									   }
						   									} else {
						   									   exclude = exclude.substring(0, start) + exclude.substring(end + 1);
						   									}
						   						}
						   			}
   		 	  }
   		 } catch(e) {
   		    debug(e, "excludeParam()");
   		 }   			
   		 
   			return exclude;
   }
   
   //************************************************/
   // Regexp document location against array
   //************************************************/
   
   function isUrlMatch(theArray) {
   			try {
			      if(theArray != null && window.content != null) {
			         var theUrl = excludeParam(window.content.document.location.href);
			         for(i = 0; i < theArray.length; i++) {
			            if(theUrl.match(theArray[i])) {
			               return true;
			            }
			         }
			      }

   		 } catch(e) {
   		    debug(e, "isUrlMatch()");
   		 }   			
   }
   
   //************************************************/
   // Encodes the String to Hex values
   //************************************************/
   
   function doHexEncode(s) {
   	  try {
			      var r = "0x";
			      var hexes = new Array("0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "a", "b", "c", "d", "e", "f");
			      for(var i = 0; i < s.length; i++) {
			         r += hexes[(s.charCodeAt(i) >> 12) & 0xf] + hexes[(s.charCodeAt(i) >> 8) & 0xf] + hexes[(s.charCodeAt(i) >> 4) & 0xf] + hexes[s.charCodeAt(i) & 0xf];
			      }
			      return r;

   		 } catch(e) {
   		    debug(e, "doHexEncode()");
   		 }   			
   }
   
   //************************************************/
   // Decodes the String to Hex values
   //************************************************/
   
   function _to_int(x1) {
   			try {
      			return x1 >= 48 && x1 < 58 ? x1 - 48 : x1 - 97 + 10;

   		 } catch(e) {
   		    debug(e, "_to_int()");
   		 }  
   }
   
   function doHexDecode(s) {
   	  try {
			      var r = "";
			      for(var i = 2; i < s.length; i += 4) {
			         x1 = _to_int(s.charCodeAt(i));
			         x2 = _to_int(s.charCodeAt(i + 1));
			         x3 = _to_int(s.charCodeAt(i + 2));
			         x4 = _to_int(s.charCodeAt(i + 3));
			         r += String.fromCharCode(((x1 << 12) & 0xF000) |((x2 << 8) & 0x0F00) |((x3 << 4) & 0x00F0) |(x4 & 0x000F));
			      }
			      return r;

   		 } catch(e) {
   		    debug(e, "doHexDecode()");
   		 }   			
   }
   
   //************************************************/
   // Debug messages
   //************************************************/

   function debug(text, method) {
   		var msg = "-------------------------------------\n" +
   												"An error within autoform has occured!\n" + 
   												"-------------------------------------\n" +
   												"Error source: actions.js\n" +
   												"Error method: " + method +"\n" +
   												"Error message: " + text + "\n" +
   												"-------------------------------------\n" +
															"Please send this details to improve the\n" +
															"quality of this extension to:\n" +
															"autoform@mozdev.org\n" +
   												"-------------------------------------\n";
						
						if (preferences.erroralert) {								
			   			alert(msg);
						} else {
							  dump(msg);
						}
   }
}
